clear all
clc
format long

%Column
d= 4.0; %diameter of column [m]
h= 7.0; %height of column - MS layer+ 2x ceramic ball layer, [m]
Qmh=681400; %Gas mass flow rate [kg/h] (**)
Qm=Qmh/3600 ; %Gas mass flow [kg/s]
channeling=0; Channeling=100*channeling; % % part of gas that's channelingif the effect of channeling (all the way through the column) is to be investigated

%Defining grids
ny=240; %number of height sections discretzed
dy=h/ny; %step size in the y-direction
tmax=60*100; %[s] Total duration of desorption (**)
dt=0.025; % [s] size of timestep (**)
nt=tmax/dt; %total number of time steps
cA=(pi()*d^2)/4; %cross sectional area of column
aA=0.62*cA; %Area the adsobent is filling (poured packing- filling factor =0.62)
fA=cA-aA; %free cross sectional area for the gas to flow

%adsorbent properties
hvap=4187; %heat of vaporisation, KJ/kg
Cpa=1.07;  %heat capacity of adsorbent, kJ/kgK
densa=720; %[kg/m3]density of adsorbent
Tadsinitial=27; %initial temperature of adsorbent (at start of regeneration)
mads= 54400; %[kg]total mass of the adsorbent in the column
dma=mads/ny; %kg adsorbent per height segment

% Making the capacity equation
P=65*10^2; %System pressure [kPa] (**)
Ppa=P*1000; %System pressure in [Pa] - to use in mwsg (Silica Gel)
R=8.314; %gas constant

%Constants for Langmuir equation follows
CH4C=[348.971 0.542 6.770E-06 13672.210; 348.971 0.542 6.130E-07 20307.220];
H2OC=[-3799.940 18.711 3.580E-07 44140.040;3684.491 -4.450 1.620E-05 45199.990];
C11=CH4C(1,1); C12=CH4C(1,2); C13=CH4C(1,3); C14=CH4C(1,4);
C21=CH4C(2,1); C22=CH4C(2,2); C23=CH4C(2,3); C24=CH4C(2,4);
H11=H2OC(1,1); H12=H2OC(1,2); H13=H2OC(1,3); H14=H2OC(1,4);
H21=H2OC(2,1); H22=H2OC(2,2); H23=H2OC(2,3); H24=H2OC(2,4);
%T is in kelvin, but as it is added 273 in the equation, C is used

%Defining MS capacity from Langmuir multicomponent adsorption (Water/C1 - MS 5A):
mwadp=@(YH2O,T) 0.725*((H2OC(1,1)/(T+273) +H2OC(1,2))*(H2OC(1,3)*exp(H2OC(1,4)/(R*(T+273))))*P*YH2O/...
(1+((H2OC(1,3)*exp(H2OC(1,4)/(R*(T+273))))*P*YH2O+(CH4C(1,3)*exp(CH4C(1,4)/(R*(T+273))))*P*(1-YH2O)))+...
(H2OC(2,1)/(T+273) +H2OC(2,2))*(H2OC(2,3)*exp(H2OC(2,4)/(R*(T+273))))*P*YH2O/(1+((H2OC(2,3)*exp(H2OC(2,4)...
/(R*(T+273))))*P*YH2O+ (CH4C(2,3)*exp(CH4C(2,4)/(R*(T+273))))*P*(1-YH2O))))*18.015/1000;
funT=@(T) 0.725*((H2OC(1,1)/(T+273) +H2OC(1,2))*(H2OC(1,3)*exp(H2OC(1,4)/(R*(T+273))))*P*YH2O/(1+((H2OC(1,3)...
*exp(H2OC(1,4)/(R*(T+273))))*P*YH2O+(CH4C(1,3)*exp(CH4C(1,4)/(R*(T+273))))*P*(1-YH2O)))+(H2OC(2,1)/(T+273) +...
H2OC(2,2))*(H2OC(2,3)*exp(H2OC(2,4)/(R*(T+273))))*P*YH2O/(1+((H2OC(2,3)*exp(H2OC(2,4)/(R*(T+273))))*P*YH2O+...
(CH4C(2,3)*exp(CH4C(2,4)/(R*(T+273))))*P*(1-YH2O))))*18.015/1000;
funT1ppm=@(T) dma*0.725*((H2OC(1,1)/(T+273) +H2OC(1,2))*(H2OC(1,3)*exp(H2OC(1,4)/(R*(T+273))))*P*10^-6/(1+...
((H2OC(1,3)*exp(H2OC(1,4)/(R*(T+273))))*P*10^-6+(CH4C(1,3)*exp(CH4C(1,4)/(R*(T+273))))*P*(1-10^-6)))+(H2OC(2,1)/(T+273)...
+H2OC(2,2))*(H2OC(2,3)*exp(H2OC(2,4)/(R*(T+273))))*P*10^-6/(1+((H2OC(2,3)*exp(H2OC(2,4)/(R*(T+273))))*P*10^-6+...
(CH4C(2,3)*exp(CH4C(2,4)/(R*(T+273))))*P*(1-10^-6))))*18.015/1000;

T=27;
yi=730*10^-6; %feed gas water content
T_initial=T;

%gas properties
hfgas=@(xx) (4*10^-6)*(xx)^3 - 0.0007*(xx)^2 +2.6643*(xx) - 66.2325;
Mdewf=@(xx) 0.000291*exp(0.039683*xx); %Water content at gas saturation as function of T
Tdewf=@(xx) 25.1997*log(3436.426*xx);%Gas temperature at water saturation as function of mwg
Cpg=2.73; %[kJ/kgK] heat capacity og the gas- the value varies very little with temperature and is therefore sat constant
CpgSI=2730; %Cpg in SI units [J/kgK]
mwg=zeros(ny,1); %matrix for amount of water in the gas- zero at start of regeneration
mwg1=mwg; %dummy matrix to be used later
densgasf=@(tt) 58.94; % [kg/m3] density of the gas as f(Tgas)
viscgasf=@(uu) 1.326E-05; %[Pa*s] -viscosity of the gas as f(Tgas)
condgasf=@(vv) 3.840E-02; % [W/m2] -thermal conductivity of gas as f(Tgas)
Mgasentering=(1-channeling)*Qm*dt; %{kg] amount of gas that enters the column every timestep
Tg=@(xx) (-2.53123*10^-13)*(xx)^5 +(4*10^-10)*(xx)^4 - (3.33*10^-7)*(xx)^3+(9*10^-5)*(xx)^2 + 0.3745*(xx) + 25; %[C]temperature of gas as a f(enthalpy)
MWgas=18.54; %MW of the gas
MWwater=18.02; %MW of water
densgasinitial=densgasf(T_initial);
densgas=densgasinitial;
vginitial=Qm/(fA*densgasinitial);   %initial velocity of the gas, m/s

%ceramic ball properties
mCpCB=5500*0.84/13; %mCp ceramic balls in one height segment- density is 0.84 kg/m3 and there are 13 height segments

%water properties
CpH2O=4.3; %[kJ/kgK] heat capacity of water. this is sat constant both for liquid and vapor water

%matrices used for calculate the gas movement upwards the column
movecells=zeros(ny,1);
moveHcells=zeros(ny,1);
densgas=zeros(ny,1);
movegascells=zeros(ny,1);
dtdivdy=dt/dy; %constant to ease code

% Defining initial conditions:
Tgas=27*ones(ny,1); %Temperature of gas in column at start of regeneration
Hgas=zeros(ny,1); %enthalpy of gas in each segment at start of regeneration
Hgas1=Hgas; %dummy matrix
Hintot=0; %to track total amount of energy entering the column
Tad=27*ones(ny,1);%initial temperature of adsorbent
Mgas=zeros(ny,1); %initial amount of gas in column
Mgas1=Mgas; %dummy matrix
Vg=zeros(ny,1); %initial velocities gas upwards the column
Energyout=zeros(ny,1); %to calculate the energy loss to surroundings
Energyouttot=zeros(ny,1); %to calculate the total amount of energy lost to surroundings
Energyin=zeros(ny,1); %to track heat transfer from bed to wall
Energyintot=zeros(ny,1); %to track total heat transfer from bed to wall
free_water=zeros(ny,1); %amount of condensed water in given height segment
ppm=0.1*ones(ny,1); %ppm in height segment. minimum ppm =0.1 for capacity function to be /=0
waterouttime=zeros((tmax/60),1); %tracking amount of gas exiting column relative to time
gasout=0; waterout=0; %tracking amount of gas and water exiting the column


%%%%%%%%CALCULATIONS%%%%%%%

minu=1; waterincounter=0;
wbar=waitbar(0,'Please wait...');   %waitbar defined

for timestep=1:nt; %timesteps
    t=dt*timestep; tm=t/60;  %t represents time [s] from start, tm [min]
    Qm=Qmh/3600;    %feedgas flowrate is at 100% throughout adsorption process, kg/s
    Mgasentering=(1-channeling)*Qm*dt; %{kg] amount of gas that enters the column every timestep
    

    for j=1:ny; %iteration in the y-direction
        
     
        %%% GAS/ADSORBTION ITERATIONS %%%
        
        %The Following creates bed saturation as a function of time and mass transfer zone (0.98m)
        MTZ_length=0.98; % Length of MTZ in [m]
        dmwater=mwadp(730*10^-6,27); %[kg] mass of water in adsorbent at initial conditions
        mwa=zeros(ny,1);
      Bed saturation as a fucntion of time
        for tmax=60*50;     %adsorption time of 50 mins
            mwa(round(5*ny/6 -2):ny)=dma*dmwater;
            for i=round(5*ny/6 -(MTZ_length*ny/7.0)):round(5*ny/6)
                sigmoid=(10/(round(5*ny/6)-round(5*ny/6 -(MTZ_length*ny/7.0)) ))*(i-round(5*ny/6 -(MTZ_length*ny/7.0)) ) -5;
                mwa(i)=dma*dmwater*(1/(1+exp(-sigmoid)));
            end
        end
        
        for tmax=60*100;     %adsorption time of 100 mins
            mwa(round(4*ny/5 -2):ny)=dma*dmwater;
            for i=round(4*ny/5 -(MTZ_length*ny/7.0)):round(4*ny/5)
                sigmoid=(10/(round(4*ny/5)-round(4*ny/5 -(MTZ_length*ny/7.0)) ))*(i-round(4*ny/5 -(MTZ_length*ny/7.0)) ) -5;
                mwa(i)=dma*dmwater*(1/(1+exp(-sigmoid)));
            end
        end
%         
        for tmax=60*150;     %adsorption time of 150 mins
            mwa(round(3*ny/4 -2):ny)=dma*dmwater;
            for i=round(3*ny/4 -(MTZ_length*ny/7.0)):round(3*ny/4)
                sigmoid=(10/(round(3*ny/4)-round(3*ny/4 -(MTZ_length*ny/7.0)) ))*(i-round(3*ny/4 -(MTZ_length*ny/7.0)) ) -5;
                mwa(i)=dma*dmwater*(1/(1+exp(-sigmoid)));
            end
        end
% %         
        for tmax=60*200;     %adsorption time of 200 mins
            mwa(round(2*ny/3 -2):ny)=dma*dmwater;
            for i=round(2*ny/3 -(MTZ_length*ny/7.0)):round(2*ny/3)
                sigmoid=(10/(round(2*ny/3)-round(2*ny/3 -(MTZ_length*ny/7.0)) ))*(i-round(2*ny/3 -(MTZ_length*ny/7.0)) ) -5;
                mwa(i)=dma*dmwater*(1/(1+exp(-sigmoid)));
            end
        end    
        
        for tmax=60*240;     %adsorption time of 240 mins
            mwa(round(1*ny/2 -2):ny)=dma*dmwater;
            for i=round(1*ny/2 -(MTZ_length*ny/7.0)):round(1*ny/2)
                sigmoid=(10/(round(1*ny/2)-round(1*ny/2 -(MTZ_length*ny/7.0)) ))*(i-round(1*ny/2 -(MTZ_length*ny/7.0)) ) -5;
                mwa(i)=dma*dmwater*(1/(1+exp(-sigmoid)));
            end
        end
    
        
        ppm(j)=(10^6*(mwg(j)*MWgas/(Mgas(j)*MWwater)));
        if isnan(ppm(j))==1 || isinf(ppm(j))==1 || ppm(j)==0 ||ppm(j)<0     %ppm must be >0 (mathematical)
            ppm(j)=0.1;
        end
        
        Tf=0.5*(Tgas(j)+Tad(j)); %arithmetic mean temperature of gas and adsorbent
        q1=0.001*dt*dy*1402.28*Qm*CpgSI*((CpgSI*viscgasf(Tf)/condgasf(Tf))^(-2/3))*...
        ((2.53*10^-4*Qm/viscgasf(Tf))^(-0.4069))*(Tgas(j)-Tad(j)); %heattransfer between gas and adsorbent from 
        %Geankoplis' relations (0.001 is to convert from J to kJ)
        qmax=0.92*(Tgas(j)-Tad(j))*(Cpg*Mgas(j)+CpH2O*mwg(j)); %max theoretical heat transfer possilble (by energy balance)
        q=min(q1,qmax); %if q1>qmax q must be sat min
        if q<0
            q=max(q1,qmax);
        end
         

        Tgasj=Tgas(j); 
        Tadj=Tad(j); 
        mwaj=mwa(j); 
        Mgasj=Mgas(j); 
        mwgj=mwg(j);   
        mwadnew=mwa(j); 
        ppmj=ppm(j);
        if abs(mwgj)<10^-12
            mwg(j)=0; mwgj=0;
        end
    end
    
    
        %specifying the inlet gas speed and H value
vg(1)=Qm/(fA*densgasinitial);      %velocity of gas in the first cell
movecells(1)=vginitial*dtdivdy; %size of cell 1 pushed upwards  
densgas(1)=densgasf(Tgas(1)); %density of gas in first cell

%calculations of gas movements upwards the column

for i=2:ny
    densgas(i)=densgasf(Tgas(i)); %density of gas in cell i
    Vg(i)=Qm/(fA*densgas(i-1));     %speed of gas cell i, calculated from properties of cell below
    movecells(i)=vg*dtdivdy;    %size of cell i that will be pushed upwards
    
    if movecells(i)>1   %if this is the case, you need to check if it might cause problems
        display('error, moving more than one cell per timestep')
        return
    end
    
    movegascells(i)=movecells(i)*Mgas(i); %how much of the mass in cell i that will move upwards
    moveHcells(i)=movecells(i)*Hgas(i); %how much of the energy that will move upwards
    movemwg(i)=movecells(i)*mwg(i);
    
end

    movegascells(1)=movecells(1)*Mgas(1); %how much of the mass in cell 1 that will move upwards
    moveHcells(1)=movecells(1)*Hgas(1); %how much of the energy in cell 1 that will move upwards
    movemwg(1)=movecells(1)*mwg(1);

for i=2:ny
        
    Mgas1(i)=Mgas(i)-movegascells(i)+movegascells(i-1); %mass of the gas within the cell i after it's shifted
    Hgas1(i)=Hgas(i)-moveHcells(i)+moveHcells(i-1); %amount of energy in gas in cell i after it's shifted
    mwg1(i)=mwg(i)-movemwg(i)+movemwg(i-1);
end

Mgasout=Mgas(ny)-movegascells(ny); Hout=Hgas(ny)-moveHcells(ny);
waterout=mwg(i); waterouttime(ceil(minu))=waterouttime(ceil(minu))+waterout;
    Mgas1(1)=Mgas(1)-movegascells(1)+Mgasentering; %amount of gas in cell one after it's shifted
%     Hgas1(1)=Hgas(1)-moveHcells(1)+Hgasentering; %amount of energy in gas in cell i after it's shifted
    mwg1(1)=mwg(1)-movemwg(1);
    Mgas=Mgas1; Hgas=Hgas1;
    mwg=mwg1;    
    for i=1:ny
        if Mgas(i)< 0.001*Mgasentering %to avoid dividing by a number close to zero
            Tgas(i)=27;
        else
            Tgas(i)=27+(Hgas(i)-(mwg(i)*(Tgas(i)-27)*CpH2O))/(Mgas(i)*Cpg);
        end
    end
    
for i=1:ny
    ppm(i)=(10^6*(mwg(i)*MWgas/(Mgas(i)*MWwater)));
    if isnan(ppm(i))==1 || isinf(ppm(i))==1 || ppm(i)==0
        ppm(i)=0.1;
    end
end

waitbar(t/tmax, wbar, timestep*dt)
end   %end of timeteps
     close(wbar)
     

     %creating plot of temperature profiles at t=tend
sam=zeros(1,1);     %matrix for adsorption, gas and wall temperatures
sam(1:1,1)=mwa(1:1,1);
aaa=(0:(7.0/(ny-1)):7.0);
plot(aaa, mwa(1:ny), 'g')
title('Adsorbent Bed saturation at t=100mins, q=681400kg/h')
xlabel('Position of column [m]')
ylabel('Adsorbent Bed Saturation, [Weight %]')
legend('Adsorption')
        
        
        




